import asyncio
import discord
from discord.ext import commands

from pydisadm.bot.adm_cog import Adm
from pydisadm.configuration import Configuration
from pydisadm.controller.adm_controller import AdmController

def create_intents():
    intents = discord.Intents.default()
    intents.message_content = True
    return intents

class AdmBot:
    def __init__(self, configuration: Configuration, controller: AdmController):
        self.configuration = configuration
        self.controller = controller

        intents = create_intents()
        self.bot = commands.Bot(application_id=configuration.discord_app_id, command_prefix=None, intents=intents, help_command=None)
    
    async def setup_cogs_async(self):
        await self.bot.add_cog(Adm(self.bot, self.configuration, self.controller))

    def setup_cogs(self):
        return asyncio.run(self.setup_cogs_async())
    
    async def sync_commands_async(self):
        await self.bot.tree.sync()

    def sync_commands(self):
        return asyncio.run(self.sync_commands_async())

    def run(self):
        self.bot.run(self.configuration.discord_token)



from importlib import resources

def get_solar_systems():
    with resources.path('pydisadm.data', 'mapSolarSystems.csv') as f:
        file_path = f
    return file_path

def get_constellations():
    with resources.path('pydisadm.data', 'mapConstellations.csv') as f:
        file_path = f
    return file_path

def get_regions():
    with resources.path('pydisadm.data', 'mapRegions.csv') as f:
        file_path = f
    return file_path



import os

class Configuration:
    """App configuration variables"""

    def __init__(self):
        self.discord_token = os.getenv('DISCORD_TOKEN')
        self.discord_channel = os.getenv('DISCORD_CHANNEL')
        self.discord_app_id = os.getenv('DISCORD_APP_ID')

        alliance_id = os.getenv('ALLIANCE_ID')
        if alliance_id != None:
            self.alliance_id = int(alliance_id)

    def __str__(self):
        return '{}(discord_token={}, discord_channel={}, discord_app_id={}, alliance_id={})'.format(
            self.__class__.__name__, self.discord_token, self.discord_channel, self.discord_app_id, self.alliance_id
        )



import requests
import logging

logger = logging.getLogger('esi_api')

def safe_get(url):
    try:
        response = requests.get(url)
    except Exception as err:
        logger.error(err)
        return None

    return response

def safe_post(url, json):
    try:
        response = requests.post(url, json=json)
    except Exception as err:
        logger.error(err)
        return None

    return response

def universe_names(ids):
    url = "https://esi.evetech.net/latest/universe/names/?datasource=tranquility"
    return safe_post(url, ids).json()

def universe_system(id):
    url = f"http://esi.evetech.net/latest/universe/systems/{id}?datasource=tranquility"
    return safe_get(url).json()

def universe_constellation(id):
    url = f"http://esi.evetech.net/latest/universe/constellations/{id}?datasource=tranquility"
    return safe_get(url).json()

def universe_region(id):
    url = f"http://esi.evetech.net/latest/universe/regions/{id}?datasource=tranquility"
    return safe_get(url).json()

def sovereignty_structures():
    url = "https://esi.evetech.net/latest/sovereignty/structures/?datasource=tranquility"
    return safe_get(url).json()



import discord

def check_allowed_channel(channel, allowed_channel):
    if allowed_channel == None:
        return True
    if isinstance(channel, discord.channel.PartialMessageable):
        return True
    if channel.name == allowed_channel:
        return True
    return 


def text_channels_with_send_permission(bot):
    for guild in bot.guilds:
        for channel in guild.channels:
            if isinstance(channel, discord.TextChannel) and channel.permissions_for(channel.guild.me).send_messages:
                yield channel
